@@ -13,36 +13,36 @@ You can use the Full OAuth flow:
 =begin code
 
     use WebService::Soundcloud;
-    
+
     my $scloud = WebService::Soundcloud.new(:$client-id, :$client-secret, redirect-uri => 'http://mydomain.com/callback' );
-    
+
     # Now get authorization url
     my $authorization_url = $scloud.get-authorization-url();
-    
+
     # Now your appplication should redirect the user to the authorization uri
     # When the User has authenticated and approved the connection they will
     # in turn be redirected back to your redirect URI with either the grant code
     # (as code) or an error (as error) as query parameters
-    
+
     # Get Access Token with the code provided as query parameter to the redirect
     my $access_token = $scloud.get-access-token($code);
-    
+
     # Save access_token and refresh_token, expires_in, scope for future use
     my $oauth_token = $access_token<access_token>;
-    
+
     # a GET request '/me' - gets users details
     my $user = $scloud.get('/me');
-    
+
     # a PUT request '/me' - updated users details
-    my $user = $scloud.put('/me', to-json( { user => { description => 'Have fun with Perl wrapper to Soundcloud API' } } ) );
-                
+    my $user = $scloud.put('/me', to-json( { user => { description => 'Have fun with the Raku wrapper to Soundcloud API' } } ) );
+
     # Comment on a Track POSt request usage
-    my $comment = $scloud.post('/tracks/<track_id>/comments', 
+    my $comment = $scloud.post('/tracks/<track_id>/comments',
                             { body => 'I love this hip-hop track' } );
-    
+
     # Delete a track
     my $track = $scloud.delete('/tracks/{id}');
-    
+
 =end code
 
 or you can use direct credential based authorisation that can skip the redirections:
@@ -65,15 +65,15 @@ or you can use direct credential based authorisation that can skip the redirecti
 
 =head1 DESCRIPTION
 
-This module provides a wrapper around Soundcloud REST API to work with 
-different kinds of soundcloud resources. It contains many functions for 
+This module provides a wrapper around Soundcloud REST API to work with
+different kinds of soundcloud resources. It contains many functions for
 convenient use rather than standard Soundcloud REST API.
 
 The complete API is documented at L<http://developers.soundcloud.com/docs>.
 
 In order to use this module you will need to register your application
 with Soundcloud at L<http://soundcloud.com/you/apps> : your application will
-be given a client ID and a client secret which you will need to use to 
+be given a client ID and a client secret which you will need to use to
 connect. The client ID used in the tests will not work correctly for your
 own application as the callback URI is set to 'localhost'.
 
@@ -116,7 +116,7 @@ It is the URI of your application that the user will be redirected
 "Connect" on the soundcloud connect page.  This will not be used if
 you are using the credential based authentication to obtain the OAuth token
 (e.g if you are an application with no UI that is operating for a single
-user.) 
+user.)
 
 =head3 method basic-params
 
@@ -244,12 +244,12 @@ L<HTTP::Response> object
 ** NOT YET IMPLEMENTED **
 
 This method is used to download a particular track id given as first argument.
-second argument is name of the destination path where the downloaded track will 
+second argument is name of the destination path where the downloaded track will
 be saved to. This method will return the file path of downloaded track.
 
 =head3 method request-format
 
-Accessor for the request format to be used.  The default is 'json' which 
+Accessor for the request format to be used.  The default is 'json' which
 should be suitable for all applications. If the format is set to something
 which Soundcloud can't deal with there will be a "406" ("Not acceptable")
 response from the API.
@@ -279,12 +279,14 @@ value.
 
 =end pod
 
-class WebService::Soundcloud:ver<0.0.8>:auth<github:jonathanstowe>:api<1.0> {
+class WebService::Soundcloud:ver<0.0.9>:auth<github:jonathanstowe>:api<1.0> {
 
     use HTTP::UserAgent;
     use URI;
     use JSON::Fast;
     use URI::Template;
+    use JSON::Name;
+    use JSON::Class;
 
     class X::NoAuthDetails is Exception {
         method message( --> Str ) {
@@ -292,6 +294,144 @@ class WebService::Soundcloud:ver<0.0.8>:auth<github:jonathanstowe>:api<1.0> {
         }
     }
 
+
+
+    class User does JSON::Class {
+        has Int     $.id;
+        has Str     $.username;
+        has Str     $.full-name                 is json-name('full_name');
+        has Str     $.description;
+        has Str     $.city;
+        has Str     $.country;
+        has Str     $.uri;
+        has Str     $.permalink;
+        has Str     $.permalink-url             is json-name('permalink_url');
+        has Str     $.website;
+        has Str     $.website-title             is json-name('website_title');
+        has Str     $.myspace-name              is json-name('myspace_name');
+        has Str     $.discogs-name              is json-name('discogs_name');
+        has Int     $.track-count               is json-name('track_count');
+        has Int     $.playlist-count            is json-name('playlist_count');
+        has Int     $.public-favorites-count    is json-name('public_favourites_count');
+        has Int     $.followings-count          is json-name('followings_count');
+        has Int     $.followers-count           is json-name('followers_count');
+        has Bool    $.online;
+    }
+
+
+    class Track does JSON::Class {
+        has Int     $.id;
+        has Str     $.uri;
+        has Str     $.video-url                 is json-name('video_url');
+        has Str     $.title;
+        has Str     $.permalink-url             is json-name('permalink_url');
+        has Str     $.waveform-url              is json-name('waveform_url');
+        has Str     $.download-url              is json-name('download_url');
+        has Str     $.attachments-uri           is json-name('attachments_uri');
+        has Str     $.created-at                is json-name('created_at');
+        has Bool    $.commentable;
+        has Bool    $.streamable;
+        has Str     $.genre;
+        has Str     $.original-format           is json-name('original_format');
+        has Int     $.download-count            is json-name('download_count');
+        has Str     $.tag-list                  is json-name('tag_list');
+        has Str     $.isrc;
+        has Int     $.bpm;
+        has Int     $.original-content-size     is json-name('original_content_size');
+        has Int     $.playback-count            is json-name('playback_count');
+        has Int     $.favoritings-count         is json-name('favoritings_count');
+        has Str     $.release;
+        has Int     $.release-day               is json-name('release_day');
+        has Int     $.release-month             is json-name('release_month');
+        has Int     $.release-year              is json-name('release_year');
+        has Str     $.description;
+        has Int     $.label-id                  is json-name('label_id');
+        has Str     $.key-signature             is json-name('key_signature');
+        has Str     $.license;
+        has Int     $.user-id                   is json-name('user_id');
+        has Str     $.sharing;
+        has Str     $.label-name                is json-name('label_name');
+        has Str     $.artwork-url               is json-name('artwork_url');
+        has Int     $.duration;
+        has Str     $.state;
+        has Str     $.permalink;
+        has Bool    $.downloadable;
+        has Str     $.purchase-url              is json-name('purchase_url');
+        has Str     $.track-type                is json-name('track_type');
+        has User    $.user;
+        has Str     $.stream-url                is json-name('stream_url');
+        has Int     $.comment-count             is json-name('comment_count');
+    }
+
+    class Playlist does JSON::Class {
+        has Int     $.id;
+        has Str     $.title;
+        has Str     $.description;
+        has Str     $.uri;
+        has Int     $.track-count       is json-name('track_count');
+        has Str     $.purchase-title    is json-name('purchase_title');
+        has Str     $.permalink-url     is json-name('permalink_url');
+        has Str     $.created-at        is json-name('created_at');
+        has Bool    $.streamable;
+        has Str     $.genre;
+        has Str     $.tag-list          is json-name('tag_list');
+        has Str     $.playlist-type     is json-name('playlist_type');
+        has Str     $.release;
+        has Int     $.release-month     is json-name('release_month');
+        has Int     $.release-day       is json-name('release_day');
+        has Int     $.release-year      is json-name('release_year');
+        has Int     $.label-id          is json-name('label_id');
+        has Str     $.type;
+        has Str     $.license;
+        has Int     $.user-id           is json-name('user_id');
+        has Str     $.sharing;
+        has Str     $.ean;
+        has Str     $.label-name        is json-name('label_name');
+        has Str     $.artwork-url       is json-name('artwork_url');
+        has Track   @.tracks;
+        has Str     $.kind;
+        has Int     $.duration;
+        has Str     $.permalink;
+        has Bool    $.downloadable;
+        has Str     $.embeddable-by     is json-name('embeddable_by');
+        has Str     $.purchase-url      is json-name('purchase_url');
+        has User    $.user;
+    }
+
+
+    class Comment does JSON::Class {
+        has Int     $.id;
+        has Str     $.body;
+        has Str     $.uri;
+        has Int     $.timestamp;
+        has Str     $.created-at    is json-name('created_at');
+        has Int     $.track-id      is json-name('track_id');
+        has Int     $.user-id       is json-name('user_id');
+        has User    $.user;
+    }
+
+    class Connection does JSON::Class {
+        has Int     $.id;
+        has Bool    $.post-publish      is json-name('post_publish');
+        has Str     $.service;
+        has Str     $.uri;
+        has Str     $.created-at        is json-name('created_at');
+        has Str     $.display-name      is json-name('display_name');
+        has Bool    $.post-favorite     is json-name('post_favorite');
+        has Str     $.type;
+    }
+
+    our constant Connections = (Array[Connection] but JSON::Class);
+
+
+    class Me is User {
+        has Str     $.plan;
+        has Bool    $.primary-email-confirmed   is json-name('primary_email_confirmed');
+        has Int     $.private-playlists-count   is json-name('private_playlists_count');
+        has Int     $.private-tracks-count      is json-name('private_tracks_count');
+    }
+
+
     # declare domains
     our %domain-for = (
         'prod'        => 'https://api.soundcloud.com/',
@@ -314,12 +454,16 @@ class WebService::Soundcloud:ver<0.0.8>:auth<github:jonathanstowe>:api<1.0> {
         'xml'  => 'application/xml'
     );
 
+    class UserAgent is HTTP::UserAgent {
+
+    }
+
 
     has Str $.client-id;
     has Str $.client-secret;
     has %!options;
     has Str $.redirect-uri is rw;
-    has HTTP::UserAgent $.ua is rw;
+    has UserAgent $.ua is rw;
     has Bool $!development = False;
     has $!scope;
     has Str $.username is rw;
@@ -329,11 +473,11 @@ class WebService::Soundcloud:ver<0.0.8>:auth<github:jonathanstowe>:api<1.0> {
     has %.auth-details;
     has Bool $!debug = False;
 
-    submethod BUILD(Str :$!client-id!, Str :$!client-secret!, Str :$!redirect-uri, Str :$!scope, Str :$!username, Str :$!password, HTTP::UserAgent :$!ua, *%opts) {
+    submethod BUILD(Str :$!client-id!, Str :$!client-secret!, Str :$!redirect-uri, Str :$!scope, Str :$!username, Str :$!password, UserAgent :$!ua, *%opts) {
 
         %!options = %opts;
         if not $!ua.defined {
-            $!ua  = HTTP::UserAgent.new;
+            $!ua  = UserAgent.new;
         }
     }
 
@@ -453,7 +597,7 @@ class WebService::Soundcloud:ver<0.0.8>:auth<github:jonathanstowe>:api<1.0> {
                 if (my $obj = self.parse-content( $res.decoded-content)).defined {
                     if $obj ~~ Array {
                         $offset += $limit;
-                        $continue = $obj.elems > 0;
+                        $continue = ( $obj.elems > 0 && $obj.elems == $limit );
                     }
                     elsif $obj ~~ Hash {
                         if $obj<collection>:exists {
@@ -606,7 +750,7 @@ class WebService::Soundcloud:ver<0.0.8>:auth<github:jonathanstowe>:api<1.0> {
         say $response.Str;
 
         if ! $response.is-success() {
-            die "Failed to fetch " 
+            die "Failed to fetch "
                 ~ $url ~ " "
                 ~ $response.content() ~ " ("
                 ~ $response.status-line() ~ ")"
@@ -653,7 +797,7 @@ class WebService::Soundcloud:ver<0.0.8>:auth<github:jonathanstowe>:api<1.0> {
         }
 
         my $uri = URI.new( $abs-url );
-   
+
         my $url-noq = $uri.Str.subst(/\?.*/,"");
 
         if  $uri.query.defined {
@@ -696,4 +840,4 @@ class WebService::Soundcloud:ver<0.0.8>:auth<github:jonathanstowe>:api<1.0> {
 
 
 }
-# vim: ft=perl6 expandtab sw=4
+# vim: ft=raku expandtab sw=4
